@@ -6,7 +6,7 @@
 # https://github.com/msberends/AMR                                     #
 #                                                                      #
 # LICENCE                                                              #
-# (c) 2018-2020 Berends MS, Luz CF et al.                              #
+# (c) 2018-2021 Berends MS, Luz CF et al.                              #
 # Developed at the University of Groningen, the Netherlands, in        #
 # collaboration with non-profit organisations Certe Medical            #
 # Diagnostics & Advice, and University Medical Center Groningen.       # 
@@ -39,6 +39,7 @@ globalVariables(c(".rowid",
                   "genus",
                   "gr",
                   "group",
+                  "guideline",
                   "hjust",
                   "input",
                   "intrinsic_resistant",
@@ -46,6 +47,7 @@ globalVariables(c(".rowid",
                   "lang",
                   "language",
                   "lookup",
+                  "method",
                   "microorganism",
                   "microorganisms",
                   "microorganisms.codes",
@@ -58,6 +60,7 @@ globalVariables(c(".rowid",
                   "old_name",
                   "pattern",
                   "R",
+                  "reference.rule",
                   "reference.rule_group",
                   "reference.version",
                   "rsi_translation",
@@ -73,5 +76,6 @@ globalVariables(c(".rowid",
                   "value",
                   "varname",
                   "xvar",
+                  "y",
                   "year",
                   "yvar"))
