@@ -6,7 +6,7 @@
 # https://github.com/msberends/AMR                                     #
 #                                                                      #
 # LICENCE                                                              #
-# (c) 2018-2020 Berends MS, Luz CF et al.                              #
+# (c) 2018-2021 Berends MS, Luz CF et al.                              #
 # Developed at the University of Groningen, the Netherlands, in        #
 # collaboration with non-profit organisations Certe Medical            #
 # Diagnostics & Advice, and University Medical Center Groningen.       # 
@@ -25,17 +25,20 @@
 
 #' Key antibiotics for first *weighted* isolates
 #'
-#' These function can be used to determine first isolates (see [first_isolate()]). Using key antibiotics to determine first isolates is more reliable than without key antibiotics. These selected isolates will then be called first *weighted* isolates.
+#' These function can be used to determine first isolates (see [first_isolate()]). Using key antibiotics to determine first isolates is more reliable than without key antibiotics. These selected isolates can then be called first *weighted* isolates.
 #' @inheritSection lifecycle Stable lifecycle
-#' @param x table with antibiotics coloms, like `AMX` or `amox`
-#' @param y,z characters to compare
+#' @param x a [data.frame] with antibiotics columns, like `AMX` or `amox`. Can be left blank when used inside `dplyr` verbs, such as `filter()`, `mutate()` and `summarise()`.
+#' @param y,z character vectors to compare
 #' @inheritParams first_isolate
-#' @param universal_1,universal_2,universal_3,universal_4,universal_5,universal_6 column names of **broad-spectrum** antibiotics, case-insensitive. At default, the columns containing these antibiotics will be guessed with [guess_ab_col()].
-#' @param GramPos_1,GramPos_2,GramPos_3,GramPos_4,GramPos_5,GramPos_6 column names of antibiotics for **Gram-positives**, case-insensitive. At default, the columns containing these antibiotics will be guessed with [guess_ab_col()].
-#' @param GramNeg_1,GramNeg_2,GramNeg_3,GramNeg_4,GramNeg_5,GramNeg_6 column names of antibiotics for **Gram-negatives**, case-insensitive. At default, the columns containing these antibiotics will be guessed with [guess_ab_col()].
-#' @param warnings give warning about missing antibiotic columns, they will anyway be ignored
-#' @param ... other parameters passed on to function
-#' @details The function [key_antibiotics()] returns a character vector with 12 antibiotic results for every isolate. These isolates can then be compared using [key_antibiotics_equal()], to check if two isolates have generally the same antibiogram. Missing and invalid values are replaced with a dot (`"."`) by [key_antibiotics()] and ignored by [key_antibiotics_equal()].
+#' @param universal_1,universal_2,universal_3,universal_4,universal_5,universal_6 column names of **broad-spectrum** antibiotics, case-insensitive. See details for which antibiotics will be used at default (which are guessed with [guess_ab_col()]).
+#' @param GramPos_1,GramPos_2,GramPos_3,GramPos_4,GramPos_5,GramPos_6 column names of antibiotics for **Gram-positives**, case-insensitive. See details for which antibiotics will be used at default (which are guessed with [guess_ab_col()]).
+#' @param GramNeg_1,GramNeg_2,GramNeg_3,GramNeg_4,GramNeg_5,GramNeg_6 column names of antibiotics for **Gram-negatives**, case-insensitive. See details for which antibiotics will be used at default (which are guessed with [guess_ab_col()]).
+#' @param warnings give a warning about missing antibiotic columns (they will be ignored)
+#' @param ... other arguments passed on to functions
+#' @details 
+#' The [key_antibiotics()] function is context-aware when used inside `dplyr` verbs, such as `filter()`, `mutate()` and `summarise()`. This means that then the `x` argument can be left blank, please see *Examples*.
+#' 
+#' The function [key_antibiotics()] returns a character vector with 12 antibiotic results for every isolate. These isolates can then be compared using [key_antibiotics_equal()], to check if two isolates have generally the same antibiogram. Missing and invalid values are replaced with a dot (`"."`) by [key_antibiotics()] and ignored by [key_antibiotics_equal()].
 #' 
 #' The [first_isolate()] function only uses this function on the same microbial species from the same patient. Using this, e.g. an MRSA will be included after a susceptible *S. aureus* (MSSA) is found within the same patient episode. Without key antibiotic comparison it would not. See [first_isolate()] for more info.
 #'
@@ -77,30 +80,30 @@
 #' # `example_isolates` is a dataset available in the AMR package.
 #' # See ?example_isolates.
 #' 
-#' # output of the `key_antibiotics` function could be like this:
+#' # output of the `key_antibiotics()` function could be like this:
 #' strainA <- "SSSRR.S.R..S"
 #' strainB <- "SSSIRSSSRSSS"
 #'
-#' # can those strings can be compared with:
+#' # those strings can be compared with:
 #' key_antibiotics_equal(strainA, strainB)
 #' # TRUE, because I is ignored (as well as missing values)
 #'
 #' key_antibiotics_equal(strainA, strainB, ignore_I = FALSE)
-#' # FALSE, because I is not ignored and so the 4th value differs
+#' # FALSE, because I is not ignored and so the 4th character differs
 #'
 #' \donttest{
 #' if (require("dplyr")) {
 #'   # set key antibiotics to a new variable
 #'   my_patients <- example_isolates %>%
-#'     mutate(keyab = key_antibiotics(.)) %>%
+#'     mutate(keyab = key_antibiotics()) %>% # no need to define `x`
 #'     mutate(
 #'       # now calculate first isolates
-#'       first_regular = first_isolate(., col_keyantibiotics = FALSE),
+#'       first_regular = first_isolate(col_keyantibiotics = FALSE),
 #'       # and first WEIGHTED isolates
-#'       first_weighted = first_isolate(., col_keyantibiotics = "keyab")
+#'       first_weighted = first_isolate(col_keyantibiotics = "keyab")
 #'     )
 #'  
-#'   # Check the difference, in this data set it results in 7% more isolates:
+#'   # Check the difference, in this data set it results in a lot more isolates:
 #'   sum(my_patients$first_regular, na.rm = TRUE)
 #'   sum(my_patients$first_weighted, na.rm = TRUE)
 #' }
@@ -127,10 +130,34 @@ key_antibiotics <- function(x,
                             GramNeg_6 = guess_ab_col(x, "meropenem"),
                             warnings = TRUE,
                             ...) {
+  if (missing(x)) {
+    x <- get_current_data(arg_name = "x", call = -2)
+  }
+  meet_criteria(x, allow_class = "data.frame")
+  meet_criteria(col_mo, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(universal_1, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(universal_2, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(universal_3, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(universal_4, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(universal_5, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(universal_6, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(GramPos_1, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(GramPos_2, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(GramPos_3, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(GramPos_4, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(GramPos_5, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(GramPos_6, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(GramNeg_1, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(GramNeg_2, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(GramNeg_3, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(GramNeg_4, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(GramNeg_5, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(GramNeg_6, allow_class = "character", has_length = 1, allow_NULL = TRUE, allow_NA = TRUE)
+  meet_criteria(warnings, allow_class = "logical", has_length = 1)
   
   dots <- unlist(list(...))
   if (length(dots) != 0) {
-    # backwards compatibility with old parameters
+    # backwards compatibility with old arguments
     dots.names <- dots %pm>% names()
     if ("info" %in% dots.names) {
       warnings <- dots[which(dots.names == "info")]
@@ -167,11 +194,11 @@ key_antibiotics <- function(x,
     }
     if (!all(col.list %in% colnames(x))) {
       if (warnings == TRUE) {
-        warning("Some columns do not exist and will be ignored: ",
-                col.list.bak[!(col.list %in% colnames(x))] %pm>% toString(),
-                ".\nTHIS MAY STRONGLY INFLUENCE THE OUTCOME.",
-                immediate. = TRUE,
-                call. = FALSE)
+        warning_("Some columns do not exist and will be ignored: ",
+                 col.list.bak[!(col.list %in% colnames(x))] %pm>% toString(),
+                 ".\nTHIS MAY STRONGLY INFLUENCE THE OUTCOME.",
+                 immediate = TRUE,
+                 call = FALSE)
       }
     }
     col.list
@@ -206,7 +233,7 @@ key_antibiotics <- function(x,
   gram_positive <- gram_positive[!is.null(gram_positive)]
   gram_positive <- gram_positive[!is.na(gram_positive)]
   if (length(gram_positive) < 12) {
-    warning("only using ", length(gram_positive), " different antibiotics as key antibiotics for Gram-positives. See ?key_antibiotics.", call. = FALSE)
+    warning_("Only using ", length(gram_positive), " different antibiotics as key antibiotics for Gram-positives. See ?key_antibiotics.", call = FALSE)
   }
   
   gram_negative <- c(universal,
@@ -215,7 +242,7 @@ key_antibiotics <- function(x,
   gram_negative <- gram_negative[!is.null(gram_negative)]
   gram_negative <- gram_negative[!is.na(gram_negative)]
   if (length(gram_negative) < 12) {
-    warning("only using ", length(gram_negative), " different antibiotics as key antibiotics for Gram-negatives. See ?key_antibiotics.", call. = FALSE)
+    warning_("Only using ", length(gram_negative), " different antibiotics as key antibiotics for Gram-negatives. See ?key_antibiotics.", call = FALSE)
   }
   
   x <- as.data.frame(x, stringsAsFactors = FALSE)
@@ -243,7 +270,7 @@ key_antibiotics <- function(x,
   key_abs <- toupper(gsub("[^SIR]", ".", gsub("(NA|NULL)", ".", x$key_ab)))
   
   if (pm_n_distinct(key_abs) == 1) {
-    warning("No distinct key antibiotics determined.", call. = FALSE)
+    warning_("No distinct key antibiotics determined.", call = FALSE)
   }
   
   key_abs
@@ -258,14 +285,20 @@ key_antibiotics_equal <- function(y,
                                   ignore_I = TRUE,
                                   points_threshold = 2,
                                   info = FALSE) {
+  meet_criteria(y, allow_class = "character")
+  meet_criteria(z, allow_class = "character")
+  meet_criteria(type, allow_class = "character", has_length = c(1, 2))
+  meet_criteria(ignore_I, allow_class = "logical", has_length = 1)
+  meet_criteria(points_threshold, allow_class = c("numeric", "integer"), has_length = 1)
+  meet_criteria(info, allow_class = "logical", has_length = 1)
+  
+  stop_ifnot(length(y) == length(z), "length of `y` and `z` must be equal")
   # y is active row, z is lag
   x <- y
   y <- z
   
   type <- type[1]
   
-  stop_ifnot(length(x) == length(y), "length of `x` and `y` must be equal")
-  
   # only show progress bar on points or when at least 5000 isolates
   info_needed <- info == TRUE & (type == "points" | length(x) > 5000)
   
